function PlayerController(id){
	var self = this; self.name = "PlayerController"; // MANDATORY

    this.dataManager = new DataManagerController();
    this.keyboard = new KeyboardController(self);
    this.userInterface = false;
    this.on_screen_alert = new OnScreenAlertController();
		
    this.renderer = null;
    this.playlist = false; // PlaylistController || TimelineController
		this.config = null;
    this.schedule = false;

    this.file_timestamp = -1;
    this.timestamp = -1;
    this.actual_dow = -1;

    this.timer = 0;

    this.registered = true;
    this.registerWindowFailRef = 666;
    this.registerWindowRegisteringRef = 777;
    this.registerWindowActivationRef = 888;
    this.alertWindowRef = 999;
    this.check_interval = 1000;

    this.scene = false;
    this.rendering = false;
		this.displayOrientation = "landscape";

    this.info_window = $('<div id="info-'+Date.now()+'" class="info">info not available yet</div>');
    this.log_window = $('<div id="log-'+Date.now()+'" class="log"></div>');
    this.ui = $('<div id="ui-'+Date.now()+'" class="ui-main"></div>');

		this.beat_timer = 0;
		
		this.id = 0;
		
		if(typeof(id) !== "undefined"){
			this.id = id;
		}

		this.init = function(base_path,force_platform){
				//var test = this.isgoingtofail(); // <-- MAKE PLAYER FAIL !! TEST !!!
        if(typeof(base_path) != "undefined"){
            self.ROUTES['BASE'].path = base_path;
        }
				self.setPlatform(force_platform);
        self.log("initialize OS ... "+self.OS.name);

        self.OS.init(self.start);
		};

    this.start = function(){
				self.log("Player version "+self.version+" Started at : "+self.getDate());
				self.notifyError(self.ERROR['PLAYER_CONTROLLER']+9,self.name+": Player version "+self.version+" Started at : "+self.getDate());
				if(typeof(LOADED_WITH_MISSING_SCRIPTS) != "undefined"){
					if(LOADED_WITH_MISSING_SCRIPTS){
						self.notifyError(self.ERROR['PLAYER_CONTROLLER']+666,self.name+": LOADED_WITH_MISSING_SCRIPTS !!! Some scripts are missing or corrupt !");
						self.OS.systemRepairUpdate();
					}
				}
				// PREPARE DISPLAY
				var display = $('<div id="stage-'+Date.now()+'" class="stage '+self.platform+' '+self.displayOrientation+'"></div>');

				$("body").append(display);
				display.append(self.info_window);
				display.append(self.log_window);
				display.append(self.ui);
				self.setInfoWindow(self.info_window);
				self.ui.on("click",self.onStageClick);
				self.log("Singleton Version : "+self.version);

				var emergency_content = self.dataManager.createArchiveFileItem(0,self.ROUTES['STYLE'].path+"logo.svg",3);
				emergency_content.relative_path = true;

				self.renderer = new DisplayController(display);
				self.userInterface = new UserInterfaceController(self.ui);
				self.renderer.init(emergency_content,self.userInterface);
				
        if(self.platform == "WEB"){
            self.loadXmlURL(self.ROUTES['BASE'].path+self.ROUTES['CONFIG'].path,self.initialConfig);
        }else if(self.platform == "SOC"){
            self.OS.readFile(self.ROUTES['CORE'].path,self.initialCore);
        }else{
            self.loadXmlURL(self.OS.rootDir+self.ROUTES['CORE'].path,self.initialCore);
        }
        if(self.beat_timer == 0){
				  self.beat();
				}
				$('.stage').css('cursor', 'none');
    };

	this.beat = function(){
			if(self.is_child){
				self.log("Child Beat !");
			}else{
				self.log("Beat !");
				clearTimeout(self.beat_timer);
				setTimeout(self.beat,7000);
        var time = self.getDate().getTime();
        time = parseInt(time/1000);
        self.OS.writeFile("admira.beat",""+time);
				try{
					self.content_api.updateInfo(self.playlist.name,self.playlist.getInfo());
				}catch(e){
					self.log("Expection : Could not update CONTENT API INFO");
				}
			}
    };

    this.onStageClick = function(){
        try{
            if(self.registered){
                if(self.config){
                    if(self.config.getQuitOnClickOption()){
                        self.OS.quit();
                    }
                }
            }
        }catch(e){
            self.log("Could not find QuitOnClick option in config tag.");
        }finally{
            self.log("Could not exit App");
        }
    };

    this.loadXmlURL = function(url,callback){
        var xhr = new XMLHttpRequest();
        url = url+"?"+Date.now();
        xhr.open("GET", url, true);
        xhr.setRequestHeader("Content-Type", "text/xml");
        xhr.addEventListener("load", function () {
            if (xhr.status === 200
                || (xhr.status === 0 && self.platform == "WEBOS")
                || (xhr.status === 0 && self.platform == "MAC")
                || (xhr.status === 0 && self.platform == "LINUX")
                || (xhr.status === 0 && self.platform == "TIZEN")
                || (xhr.status === 0 && self.platform == "WIN")
                || (xhr.status === 0 && self.platform == "ANDROID")
                || (xhr.status === 0 && self.platform == "IOS")) {
                self.log("Loaded : "+url);
                callback(xhr.response);
            }
        }, false);
        xhr.addEventListener("progress", function (e) {
            if(e.lengthComputable){
                var percent = (e.loaded / e.total) * 100;
                self.log(percent+" % of "+e.total+" bytes");
            }
        }, false);
        xhr.addEventListener("error", function (e) {
            self.log("XML Loading Error for: "+url);
            self.notifyError(self.ERROR['PLAYER_CONTROLLER']+1,self.name+": Function loadXmlURL() failed : could not load XML");
            callback();
        }, false);
        xhr.send();
    }

    this.initialCore = function(data){
        self.setAPIroutes(data,self.id);
				if(self.id != 0){self.OS.sync_master_supported = false;}
        self.ROUTES['CORE'].exists = true;
        if(self.platform == "SOC"){
						clearTimeout(self.timer);
            self.timer = setTimeout(self.getWeekTimestampFromFile,self.check_interval);
        }else{
            self.loadXmlURL(self.OS.rootDir+self.ROUTES['CONFIG'].path,self.initialConfig);
        } 
				self.content_api.updateInfo(self.name,{player_id: self.core_id, platform: self.platform});
				$(self.content_api).on("ApiContentConditionSet",self.onApiContentConditionSet);
				$(self.content_api).on("ApiContentReadFile",self.onApiContentReadFile);
				$(self.content_api).on("ApiContentWriteFile",self.onApiContentWriteFile);
				$(self.content_api).on("ApiContentAnalytic",self.onApiContentAnalytic);
				$(self.content_api).on("ApiContentMainDuration",self.onApiContentMainDuration);
				$(self.content_api).on("ApiContentGetPlayerInfo",self.onApiContentGetPlayerInfo);
				$(self.content_api).on("ApiContentPreloadNext",self.onApiContentPreloadNext);
    };

    this.initialConfig = function(data){
        self.setPaths(data);
				clearTimeout(self.timer);
        self.timer = setTimeout(self.getWeekTimestampFromXML,self.check_interval);
    };

    this.update = function(){
        if(self.platform == "WEB"){
            self.loadXmlURL(self.ROUTES['BASE'].path+self.ROUTES['CONFIG'].path);
            self.file_timestamp = -1;
            self.timestamp = -1;
        }else{
            self.file_timestamp = -1;
            self.timestamp = -1;
            self.dataManager.load(self.getDataUrl(),self.timestamp);
        }
    };

    this.getDataUrl = function(){
        var dow = self.getDayOfWeek();
        if(dow != self.actual_dow){
            self.actual_dow = dow;
            self.timestamp = -1;
            self.file_timestamp = -1;
        }
				var url = self.ROUTES['XML'].path+dow+".xml";
				if(self.id != 0){
					url = self.ROUTES['XML'].path+self.id+"_"+dow+".xml";
				}
				//self.log("Loading XML data from : "+url);
        if(self.platform == "WEB"){
            url = self.ROUTES['DATA'].path;
        //}else if(self.platform == "CHROME"){
        //    url = "persistent/"+url;
        }else{
            self.ROUTES['DATA'].path = url;
        }
        return url;
    };

    this.onDataLoaded = function(e){
        $(self.dataManager).off("DataManagerControllerDataLoaded",self.onDataLoaded);
        var new_timestamp = self.dataManager.getTimestamp();
        self.log("XML Data Loaded with Timestamp : "+new_timestamp +" Last Stamp :"+self.timestamp);
        if(self.timestamp != new_timestamp){
            self.timestamp = new_timestamp;
            self.loadConfiguration();
            self.registered = self.checkRegister();
            if(self.registered){
                var needs_init = true;
                var new_playlist = self.dataManager.getPlaylistType();
                if(!self.playlist){
                    if(new_playlist == "PlaylistController"){
                        self.playlist = new PlaylistController();
                        self.playlist.setRenderer(self.renderer);
                        $(self.playlist).one("PlaylistControllerReady",self.onPlaylistReady);
                        $(self.playlist).on("PlaylistControllerInvalidPlaylist",self.onInvalidPlaylist);
                    }else if(new_playlist == "TimelineController"){
                        self.playlist = new TimelineController();
                        self.playlist.setRenderer(self.renderer);
                        $(self.playlist).one("TimelineControllerReady",self.onPlaylistReady);
                        $(self.playlist).on("TimelineControllerInvalidPlaylist",self.onInvalidPlaylist);
                        $(self).on("SystemTimeUpdate",self.playlist.onSystemTimeUpdate);
                    }
                }else{
                    if(self.playlist.name != new_playlist){
                        $(self).off("SystemTimeUpdate",self.playlist.onSystemTimeUpdate);
                        self.playlist.unload();
                        self.playlist = null;
                        if(new_playlist == "PlaylistController"){
                            self.playlist = new PlaylistController();
                            self.playlist.setRenderer(self.renderer);
                            $(self.playlist).one("PlaylistControllerReady",self.onPlaylistReady);
                            $(self.playlist).on("PlaylistControllerInvalidPlaylist",self.onInvalidPlaylist);
                        }else if(new_playlist == "TimelineController"){
                            self.playlist = new TimelineController();
                            self.playlist.setRenderer(self.renderer);
                            $(self.playlist).one("TimelineControllerReady",self.onPlaylistReady);
                            $(self.playlist).on("TimelineControllerInvalidPlaylist",self.onInvalidPlaylist);
                            $(self).on("SystemTimeUpdate",self.playlist.onSystemTimeUpdate);
                        }
                    }else{
                        needs_init = false;
                    }
                }
								if(self.playlist){
									self.playlist.update(self.dataManager);
	                if(needs_init){
											self.renderer.clear();
	                    self.playlist.start();
	                }else{
										var jump = self.config.getAdvancedPlaylistJump();
										if(jump){
											if(self.playlist){
												if(self.playlist.active){
													self.log("XML Updated ... jump playlist forced");
													self.playlist.jump();
												}
											}
										}
									}
								}else{
									self.log("No Playlist found ... assuming MultiPlayer Parent");
								}                
            }else{
                if(self.playlist){
                    $(self.playlist).off("PlaylistControllerReady",self.onPlaylistReady);
                    self.playlist.stop();
                    self.playlist = null;
                    self.playlist = false;
                }else{
									self.log("No Playlist found ... assuming MultiPlayer Parent");
								}
            }

        }
				clearTimeout(self.timer);
        if(self.platform == "SOC"){
            self.timer = setTimeout(self.getWeekTimestampFromFile,self.check_interval);
        }else{
            self.timer = setTimeout(self.getWeekTimestampFromXML,self.check_interval);
        }
        $(self).trigger("PlayerDataLoaded");
    };

    this.checkRegister = function(){
        if(self.platform == "WEB"){return true;}
        var api = self.execute(function(){return self.config.getApiKey();});
        var activated = self.execute(function(){ return self.config.getActivationStatus();});

        var domain = self.execute(function(){ return self.config.getDomain();});

        if(!domain || domain == ""){
            domain = self.defaultDomain;
        }

        self.renderer.cancel_all(self.registerWindowFailRef,self.renderer.default_layer,true);
        self.renderer.cancel_all(self.registerWindowRegisteringRef,self.renderer.default_layer,true);
        self.renderer.cancel_all(self.registerWindowActivationRef,self.renderer.default_layer,true);
        if(activated){
            self.log("Player is registered with API KEY = "+api);
            return true;
        }else if(api){
            self.log("Player needs activation for API KEY = "+api);
            var register_content = self.dataManager.createArchiveFileItem(self.registerWindowActivationRef,"http://"+domain+"/register/registered/"+api,-1);
            register_content.relative_path = true;
            register_content.force_http_source = true;
            self.renderer.addDisplayElement(register_content);
            $(self.renderer).on("DisplayControllerElementError",self.registrationSystemFail);
            self.scene = self.renderer.prepare_scene(false,register_content.getId(),false);
            self.rendering = self.renderer.render_scene(self.scene,self.rendering);
            return false;
        }else{
            self.log("Player is initiating the registration process");
            var register_content = self.dataManager.createArchiveFileItem(self.registerWindowRegisteringRef,"http://"+domain+"/register/",-1);
            register_content.relative_path = true;
            register_content.force_http_source = true;
            self.renderer.addDisplayElement(register_content);
            $(self.renderer).on("DisplayControllerElementError",self.registrationSystemFail);
            self.scene = self.renderer.prepare_scene(false,register_content.getId(),false);
            self.rendering = self.renderer.render_scene(self.scene,self.rendering);
            return false;
        }
    };

    this.registrationSystemFail = function(e){
        $(self.renderer).off("DisplayControllerElementError",self.registrationSystemFail);
        var register_content = self.dataManager.createArchiveFileItem(self.registerWindowFailRef,self.ROUTES['STYLE'].path+"register_failed.html",-1);

        register_content.relative_path = true;
        self.renderer.addDisplayElement(register_content);
        self.scene = self.renderer.prepare_scene(false,register_content.getId(),false);
        self.rendering = self.renderer.render_scene(self.scene,self.rendering);

        self.notifyError(self.ERROR['PLAYER_CONTROLLER']+2,self.name+": Function registrationSystemFail() failed");
    };

    this.loadEvents = function(){
        var onScreenAlertsConfig = self.config.getOnScreenAlertsConfig();
        if(onScreenAlertsConfig){
            self.log("Checking onScreenAlerts ... ");
            self.on_screen_alert.update(onScreenAlertsConfig.url,onScreenAlertsConfig.interval,onScreenAlertsConfig.send_id);
            $(self.on_screen_alert).on("OnScreenAlertUpdate",self.onScreenAlertUpdate);
        }else{
            self.log("Disable check onScreenAlerts.");
            self.on_screen_alert.stop();
        }
    };

    this.loadConfiguration = function(){
        self.config = self.dataManager.getConfig();
        self.log("Loading Configuration for Player Id : "+self.config.getId());

        self.setVolume(self.config.getVolume());
        self.log("Using Volumen : "+self.config.getVolume());
        self.OS.alwaysOnTop(self.config.getAlwaysOnTop());
				if(self.platform != "WEB"){
					self.activateLogger(self.config.getDebugMode());
				}
        // REALLY NEEDED ? 
        //self.schedule = self.config.getScheduleConfig();

        var dow = self.getDayOfWeek();
        var date = self.getDate();
        var check =((dow-1)*24) + date.getHours();
        self.log("(Compare with transfer) Checking for day of week : "+dow+" on hour : "+date.getHours()+" Check hour of the week : "+check);

				// SMOOTHING NEEDS PLAYER REBOOT AFTER APPLIED
        self.renderer.setVideoProperties(self.config.getAdvancedVideoSmoothing());

        var defaultContent = self.dataManager.getArchiveFileItemById(self.config.getDefaultContent());
        defaultContent = self.execute(function(){return defaultContent;});
        self.renderer.modifyEmergencyContent(defaultContent);
        if(self.platform == "WEB"){
           self.userInterface.activate();
        }

        var info = self.config.getPlayerInfo();
        self.infoName = info.name;
        self.info_window.html("V :"+self.version+" ID : "+info.id+" Name : "+info.name+" Project : "+info.project+"<br> API KEY :"+info.api_key);

				var player_mode = self.config.getMode();
				self.setPlayerWindow();
				
				self.log("My core ID is : "+self.core_id);
				if(self.core_id == "0"){
					self.log("Setting Core ID from XML file ... "+self.config.getId());
					self.setCoreId(self.config.getId());
					self.log("No my Core ID is : "+self.core_id);
					self.log("This Core ID is : "+this.core_id);
				}
				try{
					self.loadEvents();
				}catch(e){
					self.log("Could not execute loadEvents() ");
				}
    };
		
		this.setPlayerWindow = function(){
			if(self.config){
				var screen = self.config.getScreenConfig();
				if(screen.w == "100%"){
					screen.w = window.innerWidth;
				}
				if(screen.h == "100%"){
					screen.h = window.innerHeight;
				}
				/* DEPRECATED
				
				var orientation = "landscape";
				var rotation = self.config.getAdvancedOrientation();
				if(rotation > 0 ){
					if(rotation == 90 || rotation == 270){
						orientation = "portrait";
						var temp = screen.w;
						screen.w = screen.h;
						screen.h = temp;
						screen.x = parseInt((screen.h-screen.w)/2);
						screen.y = parseInt((screen.w-screen.h)/2);
					}
					self.renderer.stage.css({
						'-webkit-transform' : 'rotate('+rotation+'deg)',
							 '-moz-transform' : 'rotate('+rotation+'deg)',
								'-ms-transform' : 'rotate('+rotation+'deg)',
								 '-o-transform' : 'rotate('+rotation+'deg)',
										'transform' : 'rotate('+rotation+'deg)',
					});
				}
				self.renderer.stage.addClass(orientation).removeClass(self.displayOrientation);
				self.displayOrientation = orientation;
				*/
				if(screen.x != 0 || screen.y != 0){
					if(self.platform == "WIN" || self.platform == "LINUX" || self.platform == "MAC"){
						screen.x = 0;
						screen.y = 0;
					}					
				}
				self.renderer.modifyViewer(screen.x,screen.y,screen.w,screen.h);
			}
		};

    this.onPlaylistReady = function(e){
        e.currentTarget.log("Info : Playlist is Ready");
        //self.loadEvents();
    };

    this.onScreenAlertUpdate = function(e){
        if(typeof(e.currentTarget.active_alert) != "undefined"){
            var active_alert = e.currentTarget.active_alert;
            if(active_alert != ""){
                self.log("Put on Screen Alert = "+active_alert);
                //if(self.playlist.active){
                    self.playlist.stop();
                    var alert_content = self.dataManager.createArchiveFileItem(self.alertWindowRef,active_alert,-1);
                    alert_content.relative_path = true;
                    alert_content.force_http_source = true;
                    self.alertWindowRef = self.renderer.addDisplayElement(alert_content);
                    $(self.renderer).on("DisplayControllerElementError",function(){
                        $(self.renderer).off("DisplayControllerElementStarted");
                        $(self.renderer).off("DisplayControllerElementError");
                        self.log("Unloading onScreenAlert.");
                        if(!self.playlist.active){
                            self.playlist.start();
                        }
                    });
                    $(self.renderer).on("DisplayControllerElementStarted",function(){
                        $(self.renderer).off("DisplayControllerElementStarted");
                        $(self.renderer).off("DisplayControllerElementError");
												self.playlist.renderer.cancel_all(self.playlist.rendering.content.reference,self.renderer.default_layer,true);
                    });
                    self.scene = self.renderer.prepare_scene(false,alert_content.getId(),false);
                    self.rendering = self.renderer.render_scene(self.scene,self.rendering);
                //}
            }else{
                self.log("Unloading onScreenAlert.");
                self.playlist.renderer.cancel_all(self.alertWindowRef,self.renderer.default_layer,true);
                if(!self.playlist.active){
                    self.playlist.start();
                }
            }
        }
    };

    this.onInvalidPlaylist = function(e){
        e.currentTarget.log("Warning : Invalid Playlist !!");
        e.currentTarget.renderer.stage.css("background","#000");
        self.notifyError(self.ERROR['PLAYER_CONTROLLER']+3,self.name+": Function onInvalidPlaylist() failed : Warning : Invalid Playlist !!",self.dataManager.getBase64XMLcopy());
    };

    this.getWeekTimestampFromFile = function(){
        clearTimeout(self.timer);
        self.OS.fileExists(self.ROUTES['XML'].path+self.ROUTES['TIMESTAMP'].path,function(exists,file_ref,params){
            if(exists){
                self.OS.readFile(self.ROUTES['XML'].path+self.ROUTES['TIMESTAMP'].path,function(data,file_ref,params){
                    self.onGetFileModificationDate(data);
                });
            }else{
                self.log("WARNING : Could not get Week Timestamp ... creating ... ");
                self.notifyError(self.ERROR['PLAYER_CONTROLLER']+4,self.name+": Function getWeekTimestampFromFile() failed : WARNING : Could not get Week Timestamp ... creating ... ");
                self.OS.writeFile(self.ROUTES['XML'].path+self.ROUTES['TIMESTAMP'].path,"1");
            }
        });

    };

    this.getWeekTimestampFromXML = function(){
        clearTimeout(self.timer);
				try{
					var url = self.getDataUrl();
					self.OS.modificationDate(url,self.onGetFileModificationDate);
	        url = null;
	        self.timer = setTimeout(self.getWeekTimestampFromXML,5000);					
				}catch(e){
					self.log("Exception trying to get Modification date ... sending 0 as modification date XML timestamp");
					self.onGetFileModificationDate(new Date().getTime());
				}
    };

    this.onGetFileModificationDate = function(modificationDate){
        clearTimeout(self.timer);
        if(modificationDate != "" && modificationDate != self.file_timestamp){
            self.log("XML Data modified on stamp : "+modificationDate);
            $(self.dataManager).one("DataManagerControllerDataLoaded",self.onDataLoaded);
            if(self.platform == "CHROMEOS"){
              var chrome_url = self.OS.fs.root.toURL()+self.getDataUrl();
              self.dataManager.load(chrome_url,self.file_timestamp);
            }else{
              self.dataManager.load(self.OS.rootDir+self.getDataUrl(),self.file_timestamp);
            }
            self.file_timestamp = modificationDate;
        }else{
			clearTimeout(self.timer);
            if(self.platform == "SOC"){
                self.timer = setTimeout(self.getWeekTimestampFromFile,self.check_interval);
            }else{
                self.timer = setTimeout(self.getWeekTimestampFromXML,self.check_interval);
            }
        }
    };

    this.setUseServerTime = function(localTime,serverTime,timezoneOffset,future_change_timestamp,future_change_offset){
        var before = self.getDate().getTime();
        self.setServerTimeParams(localTime,serverTime,timezoneOffset,future_change_timestamp,future_change_offset);
        var after = self.getDate().getTime();
        $(self).trigger("SystemTimeUpdate",[before,after]);
    };


    this.keyUp = function(keyCode,shifted){
        self.log("KeyUp : "+keyCode);
    }

    this.keyDown = function(keyCode,shifted){
        self.log("KeyDown");
        if(keyCode == 27){
            self.OS.quit();
        }
    }

    this.keyPress = function(keyCode,shifted){
        var character = String.fromCharCode(keyCode);
        self.log("KeyPressed : "+keyCode+" Key : "+character);
        if(keyCode == 108) {
            if(self.registered){
                self.log("Activate logger ... "+self.debug);
                self.debug = !self.debug;
                if(typeof(self.logger) !== 'undefined'){
                    self.setLogger(self.log_window);
                }else{
                    self.logger = null;
                }
                self.activateLogger(self.debug);
            }
        }
        if(keyCode == 100) {
            if(self.registered){
                self.log("Activate debugger ... "+self.debug);
                self.debug = !self.debug;
                self.activateLogger(self.debug);

            }
        }
        if(keyCode == 117){
            if(self.registered){
                self.update();
            }
        }
        if(keyCode == 99){
            if(self.registered){
                self.userInterface.toggle();
                self.renderer.checkUi(self.playlist.rendering.content.reference);
            }
        }
        if(keyCode == 101){
            // SIMULATE ERROR
            self.log("Simulate Error ");
            self.notifyError(1,"Simulate Error");

        }
        if(self.playlist.conditional){
            var timestamp = parseInt(self.getDate().getTime()/1000);
            var condition = {
                'id': 29,
                'filename':'keyboard.xml',
                'tstamp':timestamp,
                'version':'1.0',
                'status':'ok',
                'value':character,
								'broadcast':true
            };
            self.playlist.conditional.onNewConditionValue(condition,false);  // Use true to write condition file
        }
    }

    this.onApiContentConditionSet = function(e,command){
        if(self.playlist.conditional){
            if(typeof(command.params) !== "undefined"){
                if(typeof(command.params.id) !== "undefined"
                   && typeof(command.params.filename) !== "undefined"
                   && typeof(command.params.value) !== "undefined"){
                    var condition = {
                        'id': command.params.id,
                        'filename':command.params.filename,
                        'tstamp':parseInt(self.getDate().getTime()/1000),
                        'version':'1.0',
                        'status':'ok',
                        'value':command.params.value,
												'broadcast':true
                    };
                    self.playlist.conditional.onNewConditionValue(condition,true);  // Use true to write condition file
                }else{
                    self.notifyError(self.ERROR['PLAYER_CONTROLLER']+5,self.name+": Function onApiContentConditionSet() failed : not enough condition data");
                }
            }else{
                self.notifyError(self.ERROR['PLAYER_CONTROLLER']+5,self.name+": Function onApiContentConditionSet() failed : not valid condition data");
            }
        }else{
            self.notifyError(self.ERROR['PLAYER_CONTROLLER']+5,self.name+": Function onApiContentConditionSet() failed : not valid condition data");
        }
    };

    this.onApiContentWriteFile = function(e,command){
        if(typeof(command.params) !== "undefined"){
            if(typeof(command.params.path) !== "undefined"
               && typeof(command.params.filename) !== "undefined"
               && typeof(command.params.text) !== "undefined"){
                if(command.params.path[command.params.path.length-1] != "/"){
                    command.params.path += "/";
                }
                self.OS.writeFile(command.params.path+command.params.filename,command.params.text);
            }else{
                self.notifyError(self.ERROR['PLAYER_CONTROLLER']+6,self.name+": Function onApiContentWriteFile() failed : not enough condition data");
            }
        }else{
            self.notifyError(self.ERROR['PLAYER_CONTROLLER']+6,self.name+": Function onApiContentWriteFile() failed : not valid file data");
        }
    };

		this.onApiContentReadFile = function(e,command,client,origin){
				if(typeof(command.params) !== "undefined"){
						if(typeof(command.params.path) !== "undefined"
							 && typeof(command.params.filename) !== "undefined"
						 	 && typeof(command.params.callback) !== "undefined"){
								if(command.params.path[command.params.path.length-1] != "/"){
										command.params.path += "/";
								}
								var read_file = true;
								if(typeof(command.params.check_exists) !== "undefined"){
									if(command.params.check_exists){
											read_file = false;
											self.OS.fileExists(command.params.path+command.params.filename,function(exists,file_ref,params){
													var response = {
														file_ref: file_ref,
														data:{check_file:true,exists:exists},
														callback: command.params.callback
													}
													self.content_api.sendApiContentEvent(client,self.content_api.API_READ_FILE,origin,response);
											});
									}
								}
								if(read_file){
									self.OS.readFile(command.params.path+command.params.filename,function(data,file_ref,params){
											var response = {
												file_ref: file_ref,
												data:data,
												callback: command.params.callback
											}
											self.content_api.sendApiContentEvent(client,self.content_api.API_READ_FILE,origin,response);
									});
								}
						}else{
								self.notifyError(self.ERROR['PLAYER_CONTROLLER']+8,self.name+": Function onApiContentReadFile() failed : not enough api data");
						}
				}else{
						self.notifyError(self.ERROR['PLAYER_CONTROLLER']+8,self.name+": Function onApiContentReadFile() failed : not valid file data");
				}
		};

    this.onApiContentAnalytic = function(e,command){
        if(typeof(command.params) !== "undefined"){
            if(typeof(command.params.path) !== "undefined"
               && typeof(command.params.extension) !== "undefined"
               && typeof(command.params.json) !== "undefined"){
                var timestamp = parseInt(self.getDate().getTime()/1000);
                if(command.params.path[command.params.path.length-1] != "/"){
                    command.params.path += "/";
                }
                var path = command.params.path+self.core_id+"_"+timestamp+command.params.extension;
                self.OS.writeFile(path,JSON.stringify(command.params.json));
            }else{
                self.notifyError(self.ERROR['PLAYER_CONTROLLER']+7,self.name+": Function onApiContentAnalytic() failed : not enough condition data");
            }
        }else{
            self.notifyError(self.ERROR['PLAYER_CONTROLLER']+7,self.name+": Function onApiContentAnalytic() failed : not valid file data");
        }
    };
		
		
		this.onApiContentGetPlayerInfo = function(e,command,client,origin){
			if(typeof(command.params) !== "undefined"){
					if(typeof(command.params.callback) !== "undefined"){
						var response = {
							data:self.content_api.info,
							callback: command.params.callback
						}
						self.content_api.sendApiContentEvent(client,self.content_api.API_PLAYER_INFO,origin,response);
					}
			}
		};
		
		this.onApiContentPreloadNext = function(e,command){
			try{
				if(self.playlist.name == "PlaylistController"){
					self.log("$$$ Preload jumps : "+self.playlist.preload_jumps);
					if(self.playlist.preload_jumps <= self.playlist.elements.length){
						self.playlist.index = self.playlist.next_index;
						self.playlist.checknext(1,true);
						self.playlist.preload_jumps++;
					}else{
						self.playlist.contentToBeRendered = self.renderer.getDefaultDisplayReference();
						self.playlist.templateToBeRendered = false;
						self.playlist.scene = self.renderer.prepare_scene(self.playlist.templateToBeRendered,self.playlist.contentToBeRendered,true);
					}
				}else{
					self.log("Can not DO : onApiContentPreloadNext() ... you are on Timeline Mode");
				}
			}catch(e){
					self.log("Exception on : onApiContentPreloadNext()");
			}
		};

    this.onApiContentMainDuration = function(e,command){
        if(typeof(command.params) !== "undefined"){
            if(typeof(command.params.duration) !== "undefined" ){
                var element = self.renderer.getDisplayContentElement(self.playlist.rendering.content.reference);
                element.controller[element.active_switcher].onApiChangeDuration(e,command);
            }
        }
    };

		this.hideMouseCursor = function(){
			$('.stage').css('cursor', 'none');
			$('body').css('cursor', 'none');
			$('.player-media').css('cursor', 'none');
		};

    this.service = function(){
        if(self.registered){
            if(self.config.getAlwaysOnTop()){
                self.OS.focusWindow();
            }
            self.OS.keepAwake(self.platform);
        }
				if(self.platform == "WIN" || self.platform == "LINUX" || self.platform == "MAC" || self.platform == "ANDROID"){
					self.hideMouseCursor();
				}
    };

    if(typeof(window) !== "undefined"){
        $(window).bind("mousedown mouseup click scroll mousemove touchstart touchend",function(e){
            $(window).trigger("UserActivity");
            if(e.type == "mouseup"){
                if(self.config){
                    if(self.config.getQuitOnClick() && self.registered){
                        self.OS.quit();
                    }
                }
            }else{
                /* ------------ Mouse Condition Values  ------------
                    CLICK : String = "click"
                    MOUSE_DOWN : String = "mouseDown"
                    MOUSE_UP : String = "mouseUp"
                    MIDDLE_CLICK : String = "middleClick"
                    MIDDLE_MOUSE_DOWN : String = "middleMouseDown"
                    MIDDLE_MOUSE_UP : String = "middleMouseUp"
                    RIGHT_CLICK : String = "rightClick"
                    RIGHT_MOUSE_DOWN : String = "rightMouseDown"
                    RIGHT_MOUSE_UP : String = "rightMouseUp"
                */
                // Attention : values are lower case valid only
                if(self.playlist.conditional){
                    var timestamp = parseInt(self.getDate().getTime()/1000);

                    var mouse_value = "";
                    switch(e.which){
                        case 1: // LEFT
                            //mouse_value = "left";
                        break;
                        case 2: // MIDDLE
                            mouse_value = "middle";
                        break;
                        case 3: // RIGHT
                            mouse_value = "right";
                        break;

                    }
                    mouse_value += e.type;
                    var condition = {
                        'id': 30,
                        'filename':'mouse.xml',
                        'tstamp':timestamp,
                        'version':'1.0',
                        'status':'ok',
                        'value':mouse_value,
												'broadcast':true
                    };
                    self.playlist.conditional.onNewConditionValue(condition,false); // Use true to write condition file
                }
            }
        });
    }
}
PlayerController.prototype = new SystemController();
