function DisplayTemplate(TemplateContent,DisplayController,ConditionalController){
    var self = this; self.name = "DisplayTemplate"; // MANDATORY

    this.data = TemplateContent;
    this.renderer = DisplayController;
    this.conditional = ConditionalController;
    this.loaded = false;
    this.preloaded = false;
    this.rendering = false;
    this.reference = false;

    if(self.data){
        this.reference = self.data.getId();
    }

    this.display_tree = [];

    this.load = function(){
        self.log(" ---- LOAD TEMPLATE ---- Loaded ? "+self.loaded+" Rendering ? "+self.rendering);
        if(!self.loaded){
            if(self.data){
                self.log("Rendering with template : "+self.data.getId()+" Loaded ? "+self.loaded);
                for(var i=0;i<self.data.elements.length;i++){
                    rectangle = self.getElementScreen(i);
                    var id = false;
                    var layer = self.data.getElementSize(i).z;
                    var die = false;
                    var block = false;
                    if(self.data.elements[i].class == "ContentElement"){
                        id = self.renderer.addDisplayElement(self.data.elements[i],layer);
                        self.renderer.preload(id,layer,die,rectangle);
                        self.renderer.checkAvailability(id,layer);
                    }else if(self.data.elements[i].class == "BlockElement"){
                      try{
                        block = new DisplayBlock(self.data.elements[i],self.renderer,self.conditional);
                        block.load(false,layer,rectangle);
                      }catch(e){
                        self.log(": DisplayBlock not available --- please update the player");
                        self.notifyError(self.ERROR['DISPLAY_TEMPLATE_ELEMENT']+1,self.name+": DisplayBlock not available --- please update the player");
                      }
                    }
                    if(typeof(self.display_tree[i]) !== "undefined"){
                        self.display_tree[i] = null;
                    }
                    self.display_tree[i] = {"id":id,"layer":layer,"die":die,"rectangle":rectangle,"rendering":false,"block":block};
                    self.log("Content of template on index : "+i+" = "+id);

                }
                self.loaded = true;
            }
        }else{
            self.log("Template is already loaded");
            if(!self.rendering){
                for(var i=0;i<self.display_tree.length;i++){
                    if(!self.display_tree[i].block){
                        self.renderer.preload(self.display_tree[i].id,self.display_tree[i].layer,self.display_tree[i].die,self.display_tree[i].rectangle);
                        self.renderer.checkAvailability(self.display_tree[i].id,self.display_tree[i].layer);
                    }
                    self.display_tree[i].rendering = false;
                }
            }
        }
    };

    this.onBlockFinished = function(e){
        self.log("Block finished !!!!!!!!!!!!!!!");
        $(e.currentTarget).off("DisplayBlockFinished",self.onBlockFinished);
    };

    this.render = function(){
        self.log(" ---- RENDER TEMPLATE ---- Loaded ? "+self.loaded+" Rendering ? "+self.rendering);
        if(!self.loaded){
            self.load();
        }
        self.rendering = true;
        var rectangle = null;
        for(var i=0;i<self.display_tree.length;i++){
            rectangle = self.getElementScreen(i);
            if(!self.display_tree[i].rendering){
                if(!self.display_tree[i].block){
                    if(!self.renderer.checkAvailability(self.display_tree[i].id,self.display_tree[i].layer)){
                        self.rendering = false;
                    }else{
                        self.renderer.render(self.display_tree[i].id,self.display_tree[i].layer,self.display_tree[i].die,rectangle);
                        self.display_tree[i].rendering = true;
                    }
                }else{
                    self.display_tree[i].block.render(self.display_tree[i].layer,rectangle,false);
                    self.display_tree[i].rendering = true;
                }
            }else{
              if(!self.display_tree[i].block){
                self.renderer.refresh(self.display_tree[i].id,self.display_tree[i].layer,rectangle);
              }else{
                self.renderer.refresh(self.display_tree[i].block.next.content,self.display_tree[i].layer,rectangle);
              }
            }
        }
    };

    this.getMasterElementScreen = function(){
        //var window = {w:$(self.renderer.stage).width(),h:$(self.renderer.stage).height()};
        var window = {w:self.renderer.w,h:self.renderer.h};
        var viewer = self.convertCoordinates(window,window,{x:0,y:0,w:self.renderer.w,h:self.renderer.h});
        if(self.data){
            var patern = self.data.getMasterPaternSize();
            var rectangle = self.data.getMasterSize();
            viewer = self.convertCoordinates(window,patern,rectangle);
        }
        return viewer;
    };

    this.getElementScreen = function(index){
        //var window = {w:$(self.renderer.stage).width(),h:$(self.renderer.stage).height()};
        var window = {w:self.renderer.w,h:self.renderer.h};
        var viewer = self.convertCoordinates(window,window,{x:0,y:0,w:self.renderer.w,h:self.renderer.h});
        if(self.data){
            var patern = self.data.getMasterPaternSize();
            var rectangle = self.data.getElementSize(index);
            viewer = self.convertCoordinates(window,patern,rectangle);
        }
        return viewer;
    };

    this.convertCoordinates = function(window,patern,rectangle){
        var x_factor = window.w/patern.w;
        var y_factor = window.h/patern.h;
        if(!isNaN(rectangle.x)){rectangle.x = rectangle.x*x_factor; rectangle.x = Math.round(rectangle.x * 1000) / 1000;}
        if(!isNaN(rectangle.y)){rectangle.y = rectangle.y*y_factor; rectangle.y = Math.round(rectangle.y * 1000) / 1000;}
        if(!isNaN(rectangle.w)){rectangle.w = rectangle.w*x_factor; rectangle.w = Math.round(rectangle.w * 1000) / 1000;}
        if(!isNaN(rectangle.h)){rectangle.h = rectangle.h*y_factor; rectangle.h = Math.round(rectangle.h * 1000) / 1000;}
        return rectangle;
    };

    this.unload = function(){
        self.log(" ---- UNLOAD TEMPLATE ---- Loaded ? "+self.loaded+" Rendering ? "+self.rendering);
        if(self.data){
            /*
            for(var i=0;i<self.display_tree.length;i++){
                element = self.display_tree[i];
                self.renderer.cancel(element.id,element.layer);
            }
            */
            for(var i=0;i<self.data.elements.length;i++){
                if(typeof(self.data.elements[i]) !== "undefined"){
                    var id = false;
                    var layer = self.data.getElementSize(i).z;
                    if(self.data.elements[i].class == "ContentElement"){
                      if(typeof(self.display_tree[i]) !== "undefined"){
                        id = self.data.elements[i].getId();
                        self.renderer.cancel(id,layer,true);
                      }
                    }else if(self.data.elements[i].class == "BlockElement"){
                        if(typeof(self.display_tree[i]) !== "undefined"){
                          if(self.display_tree[i].block){
                              self.display_tree[i].block.unload();
                          }
                        }
                    }
                }
            }
        }
        self.rendering = false;
    };

    this.destroy = function(){
        /*
        self.unload();
        for(var layer in self.display_tree){
            self.display_tree[layer] = null;
        }
        self.display_tree = null;
        */
    };
}
DisplayTemplate.prototype = new SystemController();
