function BlockElement(data){
    this.class = "BlockElement"; // MANDATORY
    
    this.BLOCK_TYPE_RANDOM = 0;
    this.BLOCK_TYPE_SEQUENTIAL = 1;
    this.BLOCK_TYPE_SEQUENTIAL_MEMORY = 2;
    this.BLOCK_TYPE_AUTO = 10;
    
    this.data = data;
    this.contents = [];
    	
    this.slots = [];
    this.total = -1;
    this.position = 0;
    this.builded = false;
    this.random_array = [];
    
    this.list = this.data.find("content_block_item");
        
        
    this.getId = function(){
        return parseInt(this.data.attr('id'));
    }; 
    
    this.getContentElement = function(id){
        return this.contents[id];
    };
    
    this.getNumberOfElements = function(){
        var n_elements = this.data.find("content_block_item").length;
        return n_elements;
    };
    
    this.getTotalPlays = function(){
        if(this.total < 0){
            this.total = parseInt(this.data.attr('total'));
            if(this.total == 0){
                this.total = this.getNumberOfElements();            
            }
        }else{
            return this.total;
        }
        return this.total;
    };
    
    this.getMode = function(){
        return parseInt(this.data.attr('mode'));
    };
    
    this.getSkipMode = function(){
        if(parseInt(this.data.attr('skip')) == 1){
            return true;
        }else{
            return false;
        }
    };
    
    this.addContentElements = function(DataManager,conditional){     
        for(var i=0;i<this.list.length;i++){
            var content = DataManager.getArchiveFileItemById($(this.list[i]).attr("id"));
            if(content){
              var id = content.getId();
              if(typeof(this.contents[id]) === "undefined"){
                  this.contents[id] = content;
              }
              if(conditional){
                  var xml_conditions = this.list.eq(i).find('conditions');
                  conditions = conditional.createConditionsObject(xml_conditions); 
                  for(var j=0;j<conditions.length;j++){
                      if(conditions[j].action == 2){
                          content = DataManager.getArchiveFileItemById(conditions[j].value);
                          id = content.getId();
                          if(typeof(this.contents[id]) === "undefined"){
                              this.contents[id] = content;
                          }
                      }
                  }
              }
            }
        }
    };
    
    this.build = function(use_n_slots){
        if(!this.builded){
            this.list.sort(function(a, b){return (parseInt($(a).attr('position'))-parseInt($(b).attr('position')));});
            
            if(typeof(use_n_slots) === "undefined"){
                use_n_slots = this.getTotalPlays();
            }
            var list_index = 0;
            for(var i=0;i<use_n_slots;i++){
                if(list_index >= this.getNumberOfElements()){
                    list_index = 0;
                }
                this.slots.push({index:i,
                                content:parseInt(this.list.eq(list_index).attr('id')),
                                position:parseInt(this.list.eq(list_index).attr('position')),
                                template:parseInt(this.list.eq(list_index).attr('template_id'))});
                list_index++;            

            }
            this.position = -1;
            this.builded = true;
        }
        return this.slots;
    };
    
    this.next = function(slot_index){
        if(this.getMode() == this.BLOCK_TYPE_RANDOM){
            this.position = this.getRandomPosition(0,this.getNumberOfElements()-1,slot_index);
        }else{
            if(slot_index > 0){
              this.position=slot_index;
            }else{
              if(this.position < this.list.length){
                  this.position++;
              }else{
                  this.position=0;
              }
            }
        }
        return this.position;
    };
    
    this.getSlot = function(index){
        if(typeof(index) === "undefined"){
            index = this.position;
        }
        return this.slots[index];
    };
    
    this.updateSlot = function(index,conditional){
        if(typeof(conditional) === "undefined"){ conditional = false; }
        
        var list_index = this.next(index);
        if(typeof(this.list[list_index]) !== "undefined"){
          var contentToBeRendered = parseInt(this.list.eq(list_index).attr('id'));
          var block_item_pos = parseInt(this.list.eq(list_index).attr('position'));
          var templateToBeRendered = parseInt(this.list.eq(list_index).attr('template_id'));
          if(conditional){
              var xml_conditions = this.list.eq(list_index).find('conditions');
              conditions = conditional.createConditionsObject(xml_conditions); 
              var satisfies = conditional.checkConditions(conditions);
              if(satisfies >= 0){
                  if(conditions[satisfies].action == 2){
                      if(this.contents[parseInt(conditions[satisfies].value)] !== "undefined"){
                          contentToBeRendered = conditions[satisfies].value;
                      }
                  }else if(conditions[satisfies].action == 3){
                      templateToBeRendered = conditions[satisfies].value;
                  }
              }
          }
          
          this.slots[index] = {index:index,
                              content:contentToBeRendered,
                              position:block_item_pos,
                              template:templateToBeRendered};    
        }
        
        return this.slots[index];
    };

    this.getRandomPosition = function (min, max,slot_index) {
        if(this.random_array.length <= 0){
          for(var i=0;i<this.list.length;i++){
              this.random_array.push(i);
          }
        }        
        return this.random_array.splice(Math.floor(Math.random()*this.random_array.length),1)[0];
    };
}
