'use strict';

var node_process = require('process');
var node_main = node_process.mainModule.exports;

var player = false;

// FOR ARM ARCH
var transfer = false;
var manager = false;


var beat_timer = false;
var service_timer = false;
var multiplayer_timer = false;

var last_mouse_pos_x = 0;
var last_mouse_pos_y = 0;
var lastUserActivity = 0;
var main_win = false;

var is_multiplayer = false;
var desktop = {w:0,h:0};
var children = [];

var windows = [];


if(typeof(nw) === "undefined"){
    nw = require('nw.gui');
}

$(document).ready(function(){
  if(typeof(nw) !== "undefined"){
    main_win = nw.Window.get();
    if(main_win.title == "Admira"){
      main_win.hide();
      nw.Screen.Init();
      var screens = nw.Screen.screens;
      for(var i=0;i<screens.length;i++){
          var screen_width = screens[i].bounds.x + screens[i].bounds.width;
          if(screen_width > desktop.w){
              desktop.w = screen_width;
          }
          var screen_height = screens[i].bounds.y + screens[i].bounds.height;
          if(screen_height > desktop.h){
              desktop.h = screen_height;
          }
      }
      load_players(start_players);
    }else{
      try{
        if(!player){
            player = new PlayerController();
            $(player).on("PlayerDataLoaded",onPlayerDataLoaded);
            player.init();
            adaptPlayerWindowSize();
        }
      }catch(e){
        console.log("Unable to start player");
        update_transfer();
      }
    }
  }
});

function load_players(callback){
  var win_player_id = "";
  for(var w=0;w<windows.length;w++){
    win_player_id = windows[w].window.location.search;
    win_player_id = parseInt(win_player_id.substring(1,win_player_id.length));
    var index = children.lastIndexOf(win_player_id);
    if( index == -1){
      windows[w].close(true);
      windows.splice(w,1);
    }
  }
  children = [];
  try{
    var fs = require('fs');
    fs.readdir("xmls", function(err,files){
        if(err){
          console.log("Error for detecting XMLs");
        }else{
          var ext = "";
          var filename = "";
          for(var i=0;i<files.length;i++){
            ext = filename = files[i];
            filename = filename.substring(0,filename.lastIndexOf("."));
            ext = ext.substring(ext.lastIndexOf("."),ext.length);
            if(ext == ".xml"){
              if(filename.lastIndexOf("_") > 0){
                var id = parseInt(filename.substring(0,filename.lastIndexOf("_")));
                if(children.lastIndexOf(id) == -1){
                  children.push(id);
                  console.log("New Child Player Found with ID : "+id);
                }
              }
            }
          }
        }
        if(typeof(callback) !== "undefined"){
          callback();
        }
    });
  }catch(e){
    console.log("Unable to start players");
    start_players();
  }
}


function start_players(){
  console.log("Starting Players ...");
  var exists = false;
  var win_player_id = "";
  if(children.length > 0){
    for(var i = 0; i < children.length;i++){
      exists = false;
      for(var w=0;w<windows.length;w++){
        console.log(windows[w].window.location.search);
        win_player_id = windows[w].window.location.search;
        win_player_id = parseInt(win_player_id.substring(1,win_player_id.length));
        if(win_player_id == children[i]){
          exists = true;
          break;
        }
      }
      if(!exists){
        console.log("Starting player : "+children[i]);
        nw.Window.open("admira.html?"+children[i],{
          frame: false,
          title: "child",
          width: desktop.w,
          height: desktop.h,
          transparent: false,
          fullscreen: false,
          kiosk: false,
          show: true,
          visible_on_all_workspaces: false,
          always_on_top: true,
          resizable: false
        },function(win){
          windows.push(win);
        });
      }else{
        console.log("Player already exists");
      }
    }
    is_multiplayer = true;
  }else{
    nw.Window.open("admira.html",{
      frame: false,
      title: "player",
      width: desktop.w,
      height: desktop.h,
      transparent: false,
      fullscreen: false,
      kiosk: false,
      show: !is_multiplayer,
      visible_on_all_workspaces: false,
      always_on_top: false,
      resizable: false
    });
  }

  if(is_multiplayer && main_win.title == "Admira"){
    try{
      if(!player){
          player = new PlayerController();
          $(player).on("PlayerDataLoaded",onPlayerDataLoaded);
          player.init();
          adaptPlayerWindowSize();
      }
    }catch(e){
      console.log("Unable to start player");
      update_transfer();
    }
  }
}


function onPlayerDataLoaded(){
    adaptPlayerWindowSize();
    if(!multiplayer_timer){
        multiplayer_timer = setInterval(check_multiplayer,10000);
        if(!beat_timer){
            beat_timer = setInterval(beat,1000);
            $(window).keypress(function(event){
                //console.log("KeyPress Code = "+event.which+" Shifted = "+self.keyboardShifted);
                lastUserActivity = parseInt(player.getDate().getTime()/1000);
            });
            $(window).on("mousemove",function(event) {
                //console.log("Mouse MOVE !!");
                lastUserActivity = parseInt(player.getDate().getTime()/1000);
            });
        }
        if(!service_timer && !is_multiplayer){
            service_timer = setInterval(service,20000);
        }
    }
    update_transfer();
}

function update_transfer(){
    var archive = [];
    try{
      archive = player.dataManager.getArchiveFiles();
    }catch(e){
      console.log("No Archive !");
    }
    var schedule = undefined;
    try{
      schedule = player.config.getScheduleConfig();
    }catch(e){
      console.log("No Schedule !");
    }
    if(!player.OS.isARM){
        node_main.init();  
        node_main.update(player.timestamp,archive,player.version,player.debug,schedule,player.playlist.conditional);
    }else{
        try{
          if(!transfer){
              transfer = new TransferService();
              transfer.init(player.version);
          }
          transfer.update(player.timestamp,archive);
          transfer.activateLogger(player.debug);
        }catch(e){
          console.log("Unable to start transfer");
        }
        
        try{
          if(!manager){
              manager = new ManagerService();
              manager.init();
          }
          var schedule = undefined;
          try{
            schedule = player.config.getScheduleConfig();
          }catch(e){
            console.log("No Schedule !");
          }
          manager.update(schedule);
          manager.activateLogger(player.debug);
        }catch(e){
          console.log("Unable to start manager");
        }
    }
}

function check_multiplayer(){
  load_players(arrange_multiplayer);
}

function arrange_multiplayer(){
  // TODO : do it in case the change is in between Multiplayer and Single player and not everytime there's a multiplayer childs change
  if(windows.length != children.length){
    console.log("Multiplayer changed configuration ! reload App !");
    if(player){
      if(player.OS){
        player.OS.reload(player.platform);
      }
    }else{
      node_main.reload();
    }
  }
}



function adaptPlayerWindowSize(){
  if(player && !is_multiplayer){
    if(player.config){
      var is_fixed_size = false;
      var screen = player.config.getScreenConfig();
      if(screen.w == "100%"){
        screen.w = window.innerWidth;
      }else{
        is_fixed_size = true;
      }
      if(screen.h == "100%"){
        screen.h = window.innerHeight;
      }else{
        is_fixed_size = true;
      }
      if(is_fixed_size){
        player.OS.setMaxWindowDesktop(screen);
        player.setPlayerWindow();
      }else{
        player.OS.setMaxWindowDesktop();
        player.setPlayerWindow();
      }
    }
  }
}

function beat(){
    if(!player.OS.window_showing){
        var mouse_pos = player.OS.getMousePos();
        if(mouse_pos){
          console.log(mouse_pos);
          if(mouse_pos.x != last_mouse_pos_x || mouse_pos.y != last_mouse_pos_y){
            last_mouse_pos_x = mouse_pos.x;
            last_mouse_pos_y = mouse_pos.y;
            lastUserActivity = parseInt(player.getDate().getTime()/1000);
          }
        }
        
        var playerScreenSaverTime = parseInt(player.config.getRefreshTime());
        if(playerScreenSaverTime > 1){ // SCREENSAVER
            var now = parseInt(player.getDate().getTime()/1000);
            console.log("Now : "+now+" Last Activity : "+lastUserActivity);
            var spentTimeSinceLastActivity =  now - lastUserActivity;
            console.log("Spent Time : "+spentTimeSinceLastActivity);
            if(spentTimeSinceLastActivity > playerScreenSaverTime){
                console.log("Show Window !");
                if(!player.OS.window_showing){
                  player.OS.recover();
                  //player.OS.togglePlayerMenu();
                }
            }
        }
    }
}

function service(){
    player.service();
    var playerScreenSaverTime = parseInt(player.config.getRefreshTime());
    if(playerScreenSaverTime <= 1 && !player.OS.player_paused_by_user){
        player.OS.recover();
        //player.OS.togglePlayerMenu();
    }
    adaptPlayerWindowSize();
}
